<?php
namespace Teknisa\PLA\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla00500ConsumoPorServico extends \Teknisa\GER\Controllers\TeknisaCrud{
	protected $dataSourceName;
	protected $parameterBagColumns;
	protected $crudService;
	protected $dataSourceNameToSave = 'consserv';
	protected $parameterBag;
	protected $service;
	protected $dataSourceManager;

	public function __construct($crudService, $parameterBag, $service, $dataSourceManager){
		$this->crudService = $crudService;
		$this->parameterBag = $parameterBag;
		$this->service = $service;
		$this->dataSourceManager = $dataSourceManager;
	}

	public function findDiasemcons(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = 'pla00500_diasemcons';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'NRORG', 'DTINIMES', 'DTFINMES');
		parent::find($request, $response);
	}

	public function saveDiasemcons(DTO\Request $request, DTO\Response $response){
		$this->dataSourceNameToSave = 'diasemcons';
		$this->dataSourceName = 'diasemcons';
		parent::save($request, $response);
	}

	public function deleteDiasemcons(DTO\Request $request, DTO\Response $response){
		$this->dataSourceNameToSave = 'diasemcons';
		$this->dataSourceName = 'diasemcons';
		parent::delete($request, $response);
	}

	public function findDateConsumo(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = 'pla00500_consserv_dates';
		$this->parameterBagColumns = array('P_CDFILIAL', 'P_CDSERVFILI', 'NRORG');
		parent::find($request, $response);
	}

	public function findProduto(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = 'pla00500_produto';
		$this->parameterBagColumns = array();
		parent::find($request, $response);
	}

	public function findConsserv(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = 'pla00500_consserv';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTCONSSERV', 'NRORG', 'IDQTDECONS', 'IDTIPOCONS');
		parent::find($request, $response);
	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'consserv';
		$this->dataSourceNameToSave = 'consserv';
		parent::delete($request, $response);
	}

	public function save(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceNameToSave = 'consserv';
		$this->dataSourceName = 'consserv';
		parent::save($request, $response);
	}

	public function export(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		try{
			$produtos = $this->validaProdutos($row['P_CDPRODUTO'], $row);
			$row['P_CDPRODUTO'] = $produtos['prodValidos'];
			if($row['__sobrescrever']) {
				foreach($row['P_IDTIPOCONSGRAVA'] as $idtipocons){
					$this->service->deleteFromSubscribe($row, $idtipocons);
				}
			}
			foreach($row['P_IDTIPOCONSGRAVA'] as $idtipocons){
				$this->service->operation($row, $idtipocons);
			}
			return $response->addDataSet(new DataSet("produtosInvalidos", $produtos['prodInvalidos']));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function import(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		try{
			$produtos = $this->validaProdutos($row['P_CDPRODUTO'], $row);
			$row['P_CDPRODUTO'] = $produtos['prodValidos'];
			if($row['__sobrescrever']){
				$this->service->deleteFromSubscribe($row, $row['P_IDTIPOCONSGRAVA']);
			}
			$this->service->operation($row, $row['P_IDTIPOCONSGRAVA']);
			return $response->addDataSet(new DataSet("produtosInvalidos", $produtos['prodInvalidos']));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function validaProdutos($produtos, $params){
		$dataSourceName = 'pla00500_verificaproduto';
		$arrayProd = explode("_", $produtos);
		$prodValidos = "";
		$prodInvalidos = [];
		foreach($arrayProd as $prod){
			if($prod){
				$conditions = array (
					array (
						"columnName" => "CDFILIAL",
						"operator"   => "=",
						"value"      => $params['P_CDFILIGRAVA']
					),
					array (
						"columnName" => "CDPRODUTO",
						"operator"   => "=",
						"value"      => $prod
					)
				);
				$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
				$row = $dataSet->getRows();
				if(empty($row)) array_push($prodInvalidos, array("CDPRODUTO" => $prod, "DESCRICAO" => "PRODUTO INEXISTENTE NA UNIDADE"));
				else if($row[0]['IDPRODATIVPR'] != 'S') array_push($prodInvalidos, array("CDPRODUTO"=> $prod, "DESCRICAO" => "PRODUTO INATIVO"));
				else if($row[0]['IDPRODATIV'] != 'A') array_push($prodInvalidos, array("CDPRODUTO" => $prod, "DESCRICAO" => "PRODUTO INATIVO NA UNIDADE"));
				else $prodValidos .= "_".$row[0]['CDPRODUTO'];
			}
		}
		return array("prodValidos" => $prodValidos, "prodInvalidos" => $prodInvalidos);
	}

	public function replace(DTO\Request $request, DTO\Response $response){
		//implementar o loop que sava todas as substituições
		try{
			$produtos = $request->getDataSet()->getRows();
			$cdproddest = $produtos[(count($produtos)-1)]['CDPRODDEST'];
			$dtconsserv = $produtos[(count($produtos)-1)]['DTCONSSERV'];
			array_pop($produtos);
			foreach($produtos as $produto){
				$this->service->insertToReplace($produto, $cdproddest, $dtconsserv);
				$this->service->deleteOldProductToReplace($produto, $dtconsserv);
			}
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function findReplace(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = 'pla00500_multi_consserv';
		$this->parameterBagColumns = array('P_CDPRODUTO', 'P_DTCONSSERV', 'P_CDFILIAL', 'P_FILISERV', 'P_IDDIASEM', 'P_IDQTDECONS');
		parent::find($request, $response);
	}
	public function validaDiasemcons (DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$conditions = array (
			array (
				"columnName" => "CDFILIAL",
				"operator" 	 => "=",
				"value" 	 => $row['CDFILIAL']
			),
			array (
				"columnName" => "CDSERVFILI",
				"operator" 	 => "=",
				"value" 	 => $row['CDSERVFILI']
			),
			array (
				"columnName" => "DTSCONSSERV",
				"operator" 	 => "=",
				"value" 	 => $row['DTCONSSERV']
			),
			array (
				"columnName" => "NRORG",
				"operator" 	 => "=",
				"value" 	 => $this->crudService->getEnvironment()->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('diasemcons', $conditions);
		if(!empty($dataSet->getRows())) {
			$dia = $dataSet->getRows()[0];
			if ($dia['IDTCONSSERV'] == $row['IDQTDECONS']) {
				$dataSet = new DataSet('diasemcons', array($dia));
				$this->crudService->delete($response,$dataSet);
			} else if ($dia['IDTCONSSERV'] == 'A') {
				if ($row['IDQTDECONS'] == 'F') {
					$dia['IDTCONSSERV'] = 'P';
				} else if ($row['IDQTDECONS'] == 'P'){
					$dia['IDTCONSSERV'] = 'F';
				}
				$dataSet = new DataSet('diasemcons', array($dia));
				$newRequest = new DTO\Request\DataSet($dataSet, '', '', '');
				$rowsToSave = $this->crudService->getRowsToSave($newRequest, 'diasemcons');
				$this->crudService->save('diasemcons', $rowsToSave);
			}
		}
	}
	
	public function verificaExcecao(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$conditions = array (
			array (
				"columnName" => "CDFILIAL",
				"operator" 	 => "=",
				"value" 	 => $row['CDFILIAL']
			),
			array (
				"columnName" => "CDSERVFILI",
				"operator" 	 => "=",
				"value" 	 => $row['CDSERVFILI']
			),
			array (
				"columnName" => "DTSCONSSERV",
				"operator" 	 => "=",
				"value" 	 => $row['DTCONSSERV']
			),
			array (
				"columnName" => "NRORG",
				"operator" 	 => "=",
				"value" 	 => $this->crudService->getEnvironment()->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('diasemcons', $conditions);
		$dados = $dataSet->getRows();
		
		return $response->addDataSet(new DataSet("verificaDia", $dados));
		 
	}
	
	public function saveExcecao(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$conditions = array (
			array (
				"columnName" => "CDFILIAL",
				"operator" 	 => "=",
				"value" 	 => $row['CDFILIAL']
			),
			array (
				"columnName" => "CDSERVFILI",
				"operator" 	 => "=",
				"value" 	 => $row['CDSERVFILI']
			),
			array (
				"columnName" => "DTSCONSSERV",
				"operator" 	 => "=",
				"value" 	 => $row['DTCONSSERV']
			),
			array (
				"columnName" => "NRORG",
				"operator" 	 => "=",
				"value" 	 => $this->crudService->getEnvironment()->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('diasemcons', $conditions);
		$dados = $dataSet->getRows();
		
		if($row['IDQTDECONS'] === 'F' && $dados[0]['IDTCONSSERV'] === 'A') {
			$dados[0]['IDTCONSSERV'] =	'P';
			$this->crudService->save('diasemcons', $dados);
			$rowsToSave = $this->crudService->getRowsToSave($request, 'consserv');
			$this->crudService->save('consserv', $rowsToSave);
		}
		
		if($row['IDQTDECONS'] === 'P' && $dados[0]['IDTCONSSERV'] === 'A') {
			$dados[0]['IDTCONSSERV'] =	'F';
			$this->crudService->save('diasemcons', $dados);
			$rowsToSave = $this->crudService->getRowsToSave($request, 'consserv');
			$this->crudService->save('consserv', $rowsToSave);
		}
		
		if($row['IDQTDECONS'] === $dados[0]['IDTCONSSERV']) {
			$rowToDelete = new DataSet('diasemcons', $dados);
			$this->dataSourceManager->delete($rowToDelete);
			
			$rowsToSave = $this->crudService->getRowsToSave($request, 'consserv');
			$this->crudService->save('consserv', $rowsToSave);
		}
	}
}