<?php
namespace Teknisa\PLA\Modules\Medicao\Controllers;

use Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Pla04200ImportacaoEfetivoController {

    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $crudService;
    protected $service;
    protected $arrayMap = array(
        "FIRST",
        "COMENSAL",
        "CATRACA",
        "ANO",
        "MES",
        "DIA",
        "HORA",
        "MINUTO",
        "CLIENTE"
    );

    public function __construct($reportService, $environment, $entityManager, $dataSourceManager, $crudService, $service) {

        $this->reportService	= $reportService;
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->dataSourceManager= $dataSourceManager;
        $this->crudService      = $crudService;
        $this->service          = $service;
    }
    
    public function realizaImportacao($request, $response) {
        $row            = $request->getRow();
        $file           = $row['FILEXLS'][0]['b64File'];
        $base64         = substr($file, 23);
        $realFile       = base64_decode($base64);
        $lines          = preg_split("/(\r\n|\n|\r)/", $realFile);
        
        $paramLayout    = $this->retornaLayoutImportacao($row);
        $parameter      = explode(' ', trim($paramLayout));
        $vetorFormatado = array();
        $erros          = array();
        
        foreach($lines as $index => $line) { // percorre as linhas do txt
            $infos = preg_split('/[(\s)]+/', trim($line)); // separa as informações na linha
            // se a quantidade de informação igual a quantidade do layout parametrizado
            // ou se não tiver o cliente em alguma linha
            if(count($infos) == count($parameter) || count($infos) == count($parameter)-1) { 
                $vetorAuxiliar = array(
                    'LINHA'=> $index
                );
                foreach($infos as $key => $info) { // para cada informação na linha
                    $valor = $this->formataValores($info, $parameter[$key]);
                    $vetorAuxiliar = array_merge($vetorAuxiliar, $valor); // merge para juntar todas as informações da linha num unico array
                }
                $vetorAuxiliar = $this->adicionaDataEHora($vetorAuxiliar);
                if(!$this->dataHoraValida($vetorAuxiliar)){
                    array_push($erros, array('PARAMS' => array('LINHA'=> $index), 'MSGCODE'=> 'DATAHORA_INVALIDA'));
                }else {
                    array_push($vetorFormatado, $vetorAuxiliar);
                }
            } else {
                array_push($erros, array('PARAMS' => array('LINHA'=> $index), 'MSGCODE'=> 'FORMATO_INVALIDO'));
            }
        }
        $this->verificaDadosValidos($vetorFormatado, $row['CDFILIAL'], $erros);
        $response->addDataSet(new DataSet('efetivos', $vetorFormatado));
        $response->addDataSet(new DataSet('erros', $erros));
    }
    /*
     * Para cada dado, verifica se já tem cliente apurado naquela data para aquele serviço e unidade,
     * se tiver, adiciona a data ao log de erros
     * se não tiver, atualiza os efetivos do plenejamento, os efetivos planejados por tipo de prato
     * depois verifica se o cliente está na EFETCLIE, se tiver, faz um update, se não tiver, insere novo registro
    */
    public function importaDados($request, $response) {
        try { 
            $row = $request->getRow();
            $hasApuracao = false;
            foreach($row as $servData) {
                $clientesApurados = $this->service->verificaClienteApurado($servData);
                $apurados = array_filter($clientesApurados, function($cli) {
                    return $cli['NRAPURACFAT'] != null;
                });
                if(count($apurados) == 0) {
                    $this->service->updateEfetPla($servData);
                    $this->service->updateEfetPlaTipoPrat($servData);
                    foreach($servData['CLIENTES'] as $cliente) {
                        if(isset($cliente['CDCLIENTE'])) {
                            $hasClienteApurado = false;
                            foreach($clientesApurados as $cliApu) {
                                if($cliApu['CDCLIENTE'] == $cliente['CDCLIENTE']){
                                    $hasClienteApurado = true;
                                }
                            }
                            if($hasClienteApurado) {
                                $efetcli = array(
                                    "QTCOMEFETCLI" => $cliente['QTCOMEFETCLI'],
                                    "P_CDFILIAL"   => $servData['CDFILIAL'],
                                    "P_CDSERVFILI" => $servData['CDSERVFILI'],
                                    "P_DTCARDAPIO" => $servData['DTCARDAPIO'],
                                    "P_CDCLIENTE"  => $cliente['CDCLIENTE']
                                );
                                $this->service->updateEfetCliNaoApurados($efetcli);
                            } else {
                                $efetcli = array(
                                    "CDFILIAL"     => $servData['CDFILIAL'],
                                    "CDSERVFILI"   => $servData['CDSERVFILI'],
                                    "DTEFETCLIE"   => $servData['DTCARDAPIO'],
                                    "CDCLIENTE"    => $cliente['CDCLIENTE'],
                                    "QTCOMEFETCLI" => $cliente['QTCOMEFETCLI'],
                                    "CDSERVPLAN"   => $servData['CDSERVFILI'],
                                    "__is_new"     => true,
                                    "NRORG"        => $this->environment->getNrOrg()
                                );
                                $this->service->insertEfetCli($efetcli);
                            }
                        }
                    }
                } else {
                    // serviço já apurado na data selecionada
                    $erros = array(
                        array(
                            'MSGCODE'=> 'alreadyCalculated',
                            'PARAMS' => array(
                                'DTCARDAPIO' => $servData['DTCARDAPIO']
                            )
                        )
                    );
                    $response->addDataSet(new DataSet('erros', $erros));
                }
            }
        } catch(\Exception $e) { 
            throw $e;
        }
    }
    
    private function formataValores($informacao, $parametro) {
        $retorno = array();
        if(strpos($parametro, "8") !== false) { // se for da maneira nova, com inscrição do cliente
            $maxIndex = 8;
        } else { // se não for
            $maxIndex = 7;
        }
        for($index = 1; $index <= $maxIndex; $index ++ ) { // verifica todos os valores do parametro
            if(strpos($parametro, strval($index)) !== false) { // se tiver o indice no parametro
                $first = strpos($parametro, strval($index)); // procura a primeira posicao 
                $last = strrpos($parametro, strval($index))+1; // procura a ultima posicao 
                // caso a inscrição do cliente esteja no meio da string no layout
                // cria um array associativo fazendo cuja chave está mapeada de acordo com os valores do layout
                // e o valor é a parte da string compreendida entre a posição inicial e final encontrada no layout
                $retorno[$this->arrayMap[$index]] = substr($informacao, $first, $last-$first); 
            }
        }
        return $retorno;
    }
    
    /*
     * Adiciona data e hora formatadas na row
    */
    private function adicionaDataEHora($vetor) {
        $vetor['DATA']    = $vetor['DIA'] . "/" . $vetor['MES'] . "/20" . $vetor['ANO'];
        $vetor['HORARIO'] = $vetor['HORA'] . ":" . $vetor['MINUTO'];
        $vetor['HRSEMFORMATAR'] = $vetor['HORA'] . $vetor['MINUTO'];
        return $vetor;
    }
    
    private function verificaDadosValidos(&$dadosTxt, $cdfilial, &$erros) {
        $clientes = $this->retornaTodosClientes($cdfilial);
        $catracas = $this->retornaCatracas($cdfilial);
        foreach($dadosTxt as $key => &$row) {
            $row['CDFILIAL'] = $cdfilial;
            $this->validaERetornaComensal($row);
            $this->validaEIncluiCliente($row, $clientes, $erros);
            $this->validaEIncluiCatraca($row, $catracas, $erros);
            $this->validaEIncluiServico($row);
            if(isset($row['ERRO']) && $row['ERRO']) {
                unset($dadosTxt[$key]);
            }
        }
    }
    
    private function dataHoraValida($row) {
        return DateTime::createFromFormat('d/m/Y H:i', $row['DATA'].' '.$row['HORARIO']);
    }
    
    /*
     * Verifica se o cliente é válido, se não for, adiciona informações de erro
     * Se for válido, adiciona informações na row
    */
    private function validaEIncluiCliente(&$row, $clientes, &$erros) {
        if(isset($row['CLIENTE'])) {
            foreach($clientes as $cliente) {
                if($cliente['CDCLIENTE'] == $row['CLIENTE'] || $cliente['NRINSJURCLIE'] == $row['CLIENTE']) {
                    $row['NRINSJURCLIE'] = $cliente['NRINSJURCLIE'];
                    $row['CDCLIENTE']    = $cliente['CDCLIENTE'];
                    $row['NMRAZSOCCLIE'] = $cliente['NMRAZSOCCLIE'];
                }
            }
            if(!isset($row['CDCLIENTE'])){
                $row['ERRO'] = true;
                array_push($erros, array('PARAMS' => array('LINHA'=> $row['LINHA']), 'MSGCODE'=> 'CLIENTE_INVALIDO'));
            }
        }
    }
    
    /*
     * Verifica se o comensal é válido, se for adiciona suas informações na row
    */
    private function validaERetornaComensal(&$row) {
        $comensal = $this->verificaComensal($row['COMENSAL'], $row['CDFILIAL']);
        $row['CDCOMENSAL'] = count($comensal) > 0 ? $comensal[0]['CDCOMENSAL'] : "";
        $row['NMCOMENSAL'] = count($comensal) > 0 ? $comensal[0]['NMCOMENSAL'] : "";
    }
    
    /*
     * Verifica se a catraca é válida, se não for, adiciona informações de erro
     * Se for, adiciona suas informações na row
    */
    private function validaEIncluiCatraca(&$row, $catracas, &$erros) {
        foreach($catracas as $catraca) {
            if($catraca['CDCATRACA'] == $row['CATRACA']) {
                $row["CDCATRACA"]  = $catraca["CDCATRACA"];
                $row["DSCATRACA"]  = $catraca["DSCATRACA"];
                $row["CDSERVFILI"] = $catraca["CDSERVFILI"];
                $row["NMSERVICO"]  = $catraca["NMSERVICO"];
                $row["HRFINASERV"] = $catraca["HRFINASERV"];
                $row["HRINICSERV"] = $catraca["HRINICSERV"];
            }
        }
        if(!isset($row['CDCATRACA'])){
            $row['ERRO'] = true;
            array_push($erros, array('PARAMS' => array('LINHA'=> $row['LINHA']), 'MSGCODE'=> 'CATRACA_INVALIDA'));
        }
    }
    
    /*
     * Verifica se tem código do serviço na row, se não tiver, roda um select pegando o primeiro serviço,
     * se tiver, adiciona informações do cardápio na row
     * por fim, valida se o horário informado no txt está entre o horario inicial e final do serviço
    */
    private function validaEIncluiServico(&$row){
        if(!isset($row['CDSERVFILI']) || $row['CDSERVFILI'] == null) {
            $cliente = isset($row['CDCLIENTE']) ? $row['CDCLIENTE'] : null;
            $servicos = $this->retornaServico($row['CDFILIAL'], $row['DATA'], $row['HRSEMFORMATAR'], $cliente);
            if(count($servicos) > 0) {
                $row['CDSERVFILI'] = $servicos[0]['CDSERVFILI'];
                $row['NMSERVICO']  = $servicos[0]['NMSERVICO'];
                $row['HRINICSERV'] = $servicos[0]['HRINICSERV'];
                $row['HRFINASERV'] = $servicos[0]['HRFINASERV'];
                $row['NRCARDAPIO'] = $servicos[0]['NRCARDAPIO'];
            }
        } else {
            $cardapio = $this->service->verificaCardapio($row); //se tiver cardápio
            if($cardapio[0] && $cardapio[0]['NRCARDAPIO']){
                $row['NRCARDAPIO'] = $cardapio[0]['NRCARDAPIO'];
            } else {
                $row['CDSERVFILI'] = "";
                $row['NMSERVICO']  = "";
                $row['HRINICSERV'] = "";
                $row['HRFINASERV'] = "";
                $row['NRCARDAPIO'] = "";
            }
        }
        $this->validaIntervaloHorario($row);
    }
    
    /*
    * Valida se os a hora do txt está entre os 
    * horários de inicio e fim do serviço
    */
    private function validaIntervaloHorario(&$row) {
        if(isset($row['HRINICSERV']) && isset($row['HRINICSERV'])) {
            
            if($row['HRSEMFORMATAR'] < $row['HRINICSERV']  || $row['HRSEMFORMATAR'] > $row['HRFINASERV']) {
                $row['CDSERVFILI'] = "";
                $row['NMSERVICO']  = "";
                $row['HRINICSERV'] = "";
                $row['HRFINASERV'] = "";
                $row['NRCARDAPIO'] = "";
            }
        }
    }
    
    private function retornaLayoutImportacao($row) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramplaneja', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $paramplaneja = $dataSet->getRows();
        return $paramplaneja[0]['TXLAYOUTCATR'];
    }
    
    private function retornaTodosClientes($cdfilial) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        
        $dataSet = $this->crudService->getFilteredDataset('pla04200_inscricao_cliente', $conditions);
        return $dataSet->getRows();
    }
    
    private function verificaComensal($cdcomensal, $cdfilial) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "CDCOMENSAL",
                "operator" => "=",
                "value" => $cdcomensal
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        
        $dataSet = $this->crudService->getFilteredDataset('pla04200_verifica_comensal', $conditions);
        return $dataSet->getRows();
    }
    
    private function retornaCatracas($cdfilial) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        
        $dataSet = $this->crudService->getFilteredDataset('pla04200_retorna_catracas', $conditions);
        return $dataSet->getRows();
    }
    
    public function retornaServico($cdfilial, $dtcardapio, $hrentrada, $cdcliente = null) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "DTCARDAPIO",
                "operator" => "=",
                "value" => $dtcardapio
            ),
            array(
                "columnName" => "CDCLIENTE",
                "operator" => "=",
                "value" => $cdcliente
            ),
            array(
                "columnName" => "HRENTRADA",
                "operator" => "=",
                "value" => $hrentrada
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        
        $dataSet = $this->crudService->getFilteredDataset('pla04200_servicos_catraca', $conditions);
        return $dataSet->getRows();
    }
}