<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;


class Pla02400SetorDoCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $dataSourceName;
	protected $parameterBagColumns;
	protected $dataSourceNameToSave;
	protected $crudService;
	protected $dataSourceManager;
	protected $parameterBag;
	protected $environment;

	public function __construct($crudService, $parameterBag, $dataSourceManager, $environment){
		$this->crudService 			 = $crudService;
		$this->parameterBag      = $parameterBag;
		$this->dataSourceManager = $dataSourceManager;
		$this->environment 			 = $environment;

		parent::__construct($crudService, $parameterBag);
	}

	public function findPadrao($request, $response){
		$this->dataSourceName = 'pla02400_padrao';
		$this->parameterBagColumns = array('CDFILIAL', 'NRORG');
		parent::find($request, $response);
	}

	public function savePadrao($request, $response){
		$this->dataSourceNameToSave = 'setotipoprat';
		parent::save($request, $response);
	}

	public function findExcecoes($request, $response){
		$this->dataSourceName = 'pla02400_excecoes';
		parent::find($request, $response);
	}


	public function saveExcecoes($request, $response){
		try {
			$row = $request->getDataSet()->getRows()[0];
			$conditions = array(
				array(
					'columnName' => 'CDFILIAL',
					'operator' => '=',
					'value' => $row['CDFILIAL']
				),
				array(
					'columnName' => 'CDTIPOPRAT',
					'operator' => '=',
					'value' => $row['CDTIPOPRAT']
				),
				array(
					'columnName' => 'CDPRODUTO',
					'operator' => '=',
					'value' => $row['CDPRODUTO']
				),
				array(
					'columnName' => 'CDSETORCOZ',
					'operator' => '=',
					'value' => $row['CDSETORCOZ']
				)
			);

			$this->verifyExistenceDietapadr($row['CDDIETPAD'], $conditions);
			$row['NRSEQSETOPRC'] = $this->findNvlMaxExcecoes($conditions);
			$this->dataSourceNameToSave = 'setoprodcard';
			parent::save($request, $response);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function verifyExistenceDietapadr($cddieta, $conditions){
		$dietapadr;
		if(empty($cddieta)){
			$dietapadr = array(
				'columnName' => 'CDDIETPAD',
				'operator' => 'IS NULL OR CDDIETPAD = ',
				'value' => NULL
			);
		} else {
			$dietapadr = array(
				'columnName' => 'CDDIETPAD',
				'operator' => '=',
				'value' => $cddieta
			);
		}
		array_push($conditions, $dietapadr);

		$filterCriteria = new FilterCriteria('setoprodcard', $conditions);
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$rows = $this->dataSourceManager->findBy($filterCriteria)->getRows();

		array_pop($conditions);

		if(!empty($rows)){
			throw new \Exception("Impossível adicionar. Registro já existe.");
		}
	}

	public function findNvlMaxExcecoes($conditions){
		$filterCriteria = new FilterCriteria('pla02400_max_nrseqsetoprc', $conditions);
		$parameterBagColumns = array('CDFILIAL','CDSETORCOZ', 'CDTIPOPRAT', 'CDPRODUTO', 'NRORG');
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $parameterBagColumns);
		$rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
		if(empty($rows) && empty($rows[0])){
			return '01';
		} else {
			$sequencial = intval($rows[0]['NRSEQSETOPRC']) + 1;
			if($sequencial < 10) {
				return '0'.strval($sequencial);
			} return strval($sequencial);
		}
	}

	public function deleteExcecoes($request, $response){
		$this->dataSourceNameToSave = 'setoprodcard';
		$this->dataSourceName = 'setoprodcard';
		parent::delete($request, $response);
	}

	public function findExcecoesTransportadas($request, $response){
		$this->dataSourceName = 'pla02400_excecoes_transportadas';
		parent::find($request, $response);
	}
	public function saveExcecoesTransportadas($request, $response){
		$this->dataSourceNameToSave = 'setoprodcardtrans';
		parent::save($request, $response);
	}
	public function deleteExcecoesTransportadas($request, $response){
		$this->dataSourceName = 'setoprodcardtrans';
		parent::delete($request, $response);
	}
}